from typing import Dict, Optional, Union
from .transfer import transfer_to_dict



all  = [
    "to_latex",
    "to_string",
    "to_arc",
    "to_arrow",
    "to_arrow_down",
    "to_equation",
    "to_fraction",
    "to_latexarray",
    "to_latexwrap",
    "to_limit",
    "to_average",
    "to_repeater",
    "to_repeating",
    "to_root",
    "to_script",
    "to_sum",
    "to_underbrace",
    "to_underline",
    "to_valence",
    "to_vector",
    "to_font",
    "to_img",
    "to_list",
    "to_matrix",
    "to_richtext",
    "to_select",
    "to_multiselect",
    "to_table",
    "to_text",
    "to_bold",
    "to_italic",
    "to_unit",
    "to_merge",
    "to_wave"
]


def lls():
    print("lls")

def to_latex(data: Dict[str, Union[str, Dict]]) -> str:
    return str(data)


def to_string(data):
    return data["value"]


def to_arc(data: Dict[str, str]) -> str:
    data = transfer_to_dict(data)
    return "\\overset{\\LARGE{\\frown}}{%s}" % data["value"]


def to_arrow(data: Dict[str, Dict[str, str]]) -> str:
    data = transfer_to_dict(data)
    top = data["value"]["top"]
    bottom = data["value"]["bottom"]
    return f"\\xrightarrow[{bottom}]{{{top}}}"


def to_arrow_down(data: Dict[str, Dict[str, str]]) -> str:
    data = transfer_to_dict(data)
    left_lines = data["value"]["left"].split("[br]")
    right_lines = data["value"]["right"].split("[br]")

    total_lines = max(len(left_lines), len(right_lines))
    middle_row = total_lines // 2

    rows = []
    for i in range(total_lines):
        l = left_lines[i] if i < len(left_lines) else ""
        r = right_lines[i] if i < len(right_lines) else ""
        arrow = "\\downarrow" if i == middle_row else ""
        rows.append(f"{l} & {arrow} & {r} \\\\")

    return f"""
    \\begin{{array}}{{lcl}}
    {''.join(rows)}
    \\end{{array}}
    """


def to_equation(data):
    from ..converter.converter import converter
    data = transfer_to_dict(data)
    
    equations = "\\\\".join([converter(item) for item in data["value"]])
    left_brace = ""
    right_brace = ""

    # 根据 "direction" 来决定添加哪种括号
    direction = data.get("direction", "left")
    if direction == "left":
        left_brace = "\\left\\{"
        right_brace = "\\right."
    elif direction == "right":
        left_brace = "\\left."
        right_brace = "\\right\\}"
    elif direction == "both":
        left_brace = "\\left\\{"
        right_brace = "\\right\\}"
    elif direction == "parentheses_left":
        left_brace = "\\left("
        right_brace = "\\right."
    elif direction == "parentheses_right":
        left_brace = "\\left."
        right_brace = "\\right)"
    elif direction == "parentheses_both":
        left_brace = "\\left("
        right_brace = "\\right)"

    return f"{left_brace} \\begin{{aligned}} {equations} \\end{{aligned}} {right_brace}"


def to_fraction(data):
    data = transfer_to_dict(data)
    value = data.get("value")
    from ..converter.converter import c2
    
    # 检查value是否为空或者是否不包含numerator和denominator
    if not value or not value.get("numerator") or not value.get("denominator"):
        return "\\underline{\\hspace{1cm}} \\bigg/ \\underline{\\hspace{1cm}}"
 
    numerator = data.get("value").get("numerator")
    if not isinstance(numerator, str):
        numerator = c2(numerator)
    denominator = data.get("value").get("denominator")
    print(f"den is {denominator}")
    if not isinstance(denominator, str):
        denominator = c2(denominator)
    
    return f"\\frac{{{numerator}}}{{{denominator}}}"


def to_latexarray(data):
    data = transfer_to_dict(data)
    rows = []
    left_brace = "\\left\\{"
    right_brace = "\\right."
    if data["type"] == "latexarray":
        value_list = data["value"]
        print(f"start c")
        for value in value_list:
            if len(value) == 1:
                print(f"value is {value} at len 1")
                print(f"type of value is {type(value)}")
                if isinstance(value, str):
                    converted_row = value
                    rows.append(" & ".join(converted_row))
                else:
                    for v in value:
                        if isinstance(v, str):
                            converted_row = v
                            rows.append(converted_row)
                        else:
                            for k in v:
                                from ..converter.converter import converter
                                print(f" 2v is {k}")
                                converted_row = k
                                rows.append(converter(converted_row))
                                # from ..converter.converter import converter
                                # print(f" 2v is {v}")
                                # converted_row = v
                                # rows.append(converter(converted_row))
                        print(f"rows is {rows}")
            else:
                temp_str = ""
                for v in value:
                    if isinstance(v, str):
                        temp_str += v
                    # Todo: recursion
                    else:
                        if v["type"] == "fraction":
                            numerator = v["value"]["numerator"]
                            denominator = v["value"]["denominator"]
                            temp_str += f"\\frac{{{numerator}}}{{{denominator}}}"
                converted_row = temp_str

                rows.append(temp_str)
        s = "\\begin{{array}}{{c}} {} \\end{{array}}".format(" \\\\ ".join(rows))
        print(f"final is {s}")
        return f"{left_brace} \\begin{{aligned}} {s}\\end{{aligned}} {right_brace}"


def to_latexwrap(data):
    data = transfer_to_dict(data)
    rows = []
    left_symbol = data.get("left", "{")
    right_symbol = data.get("right", ".")
    print("sss")

    # 根据 left_symbol 和 right_symbol 选择适当的 LaTeX 符号
    if left_symbol == "{":
        left_brace = "\\left\\{"
    elif left_symbol == "(":
        left_brace = "\\left("
    else:
        left_brace = "\\left."

    if right_symbol == "}":
        right_brace = "\\right\\}"
    elif right_symbol == "(":
        right_brace = "\\right("
    elif right_symbol == ")":
        right_brace = "\\right)"
    else:
        right_brace = "\\right."
    print(f"data is {data['value']}")

    print("pr", to_latexarray(data["value"]))
    # for value in data["value"]:
        
    #     print(f"value is {value}")
    #     if isinstance(value, str):
    #         printlatexarray(f" is str {value}")
    #         rows.append(value)
    #     else:
    #         type = value["type"]
    #         if type == "latexarray":
    #             latex_col = (
    #                 " \\begin{array}{c} "
    #                 + " \\\\ ".join(v[0] for v in value["value"])
    #                 + " \\end{array} "
    #             )
    #             rows.append(latex_col)
                
    latex_aligned = "\\left\\{ \\begin{aligned}"
    s = to_latexarray(data["value"])
    print(f"s is {s}")
    
    if latex_aligned in s:
        # If it does, return 's' as it is
        return s
    else:
        # Otherwise, format 's' within an aligned block and return
        ni = f"\\left\\{{ \\begin{{aligned}} {s} \\end{{aligned}} \\right."
        print(f"ni is {ni}")
        return ni
   
    print(f"s is {s}")
    ni = f"{left_brace} \\begin{{aligned}} {s}\\end{{aligned}} {right_brace}"
    print(f"ni is {ni}")
    return f"{left_brace} \\begin{{aligned}} {s}\\end{{aligned}} {right_brace}"


def to_limit(data):
    data = transfer_to_dict(data)
    lower = data["value"]["lower"]
    function = data["value"]["function"]
    return f"\\lim_{{{lower}}} {function}"


def to_average(data):
    data = transfer_to_dict(data)
    value = data["value"]
    return f"\\overline{{{value}}}"


def to_repeater(data):
    data = transfer_to_dict(data)
    value = data["value"]
    # 使用 \dot 来为数字添加点
    return f"\\dot{{{value}}}"


def to_repeating(data):
    data = transfer_to_dict(data)
    if data["type"] == "repeating":
        pre = data["value"]["pre"]
        repeater = data["value"]["repeater"]

        # 根据 repeater 的长度，决定如何为数字添加点
        if len(repeater) == 1:
            repeater_latex = f"\\dot{{{repeater}}}"
        elif len(repeater) == 2:
            repeater_latex = f"\\dot{{{repeater[0]}}}\\dot{{{repeater[1]}}}"
        else:
            repeater_latex = (
                f"\\dot{{{repeater[0]}}}{repeater[1:-1]}\\dot{{{repeater[-1]}}}"
            )

        return f"0.{pre}{repeater_latex}"


def fraction_to_latex(data):
    if not isinstance(data, dict):
        data = transfer_to_dict(data)
    numerator_content = []
    for item in data["value"]["numerator"]:
        if isinstance(item, str):
            numerator_content.append(item)
        elif item["type"] == "script":
            numerator_content.append(to_script(item))
    numerator = "".join(numerator_content)
    denominator = data["value"]["denominator"]
    return f"\\displaystyle \\frac{{{numerator}}}{{{denominator}}}"


def script_to_latex(data):
    if not isinstance(data, dict):
        data = transfer_to_dict(data)
    front = data["value"]["front"]
    sub = data["value"]["sub"]
    return f"{front}_{{{sub}}}"


def to_root(data):
    from ..converter.converter import c2
    data = transfer_to_dict(data)


    radicand = data["value"].get("radicand")
    # print(f"error on {radicand}")
    # print(f"data is {radicand}")
    # print("type os radicand is ", type(radicand))

    if isinstance(radicand, str):
        return f"\\displaystyle \\sqrt{{{radicand}}}"

    if data["type"] == "root":
        index = data["value"].get("index", "")
        if index:

            numerator  = f"[{index}]"
            denominator = c2(radicand)
        else:

            print(radicand[0], type(radicand[0]))
            numerator = c2(radicand[0])

            denominator = c2(radicand[1])

        
        
        

        return f"\\displaystyle \\sqrt{{{numerator}{{{denominator}}}}}"


def to_script(data):
    data = transfer_to_dict(data)
    front = data["value"].get("front", "")
    sup = data["value"].get("sup", "")
    sub = data["value"].get("sub", "")

    if sup and sub:  # 同时存在上标和下标
        return f"{front}^{{{sup}}}_{{{sub}}}"
    elif sup:  # 只存在上标
        return f"{front}^{{{sup}}}"
    elif sub:  # 只存在下标
        return f"{front}_{{{sub}}}"
    else:  # 不应该出现既没有上标也没有下标的情况
        return front


def to_sum(data):
    data = transfer_to_dict(data)
    upper_bound = data["value"]["upper"]["value"]
    lower_bound = data["value"]["lower"]["value"]
    variable = data["value"]["variable"]

    return f"\\displaystyle \\sum_{{{lower_bound}}}^{{{upper_bound}}} {variable}"


def to_underbrace(data):
    data = transfer_to_dict(data)
    top_data = data["value"]["top"]["value"]
    top_elements = []
    for item in top_data:
        if item["type"] == "string":
            top_elements.append(item["value"])
        elif item["type"] == "script":
            top_elements.append(to_script(item))
    top_string = "".join(top_elements)

    # 获取bottom部分的值
    bottom = data["value"]["bottom"]

    return f"\\displaystyle \\underbrace{{{top_string}}}_{{\\text{{{bottom}}}}}"


def to_underline(data):
    data = transfer_to_dict(data)
    value = data["value"]
    return f"\\displaystyle \\underline{{{value}}}"


def to_valence(data):
    data = transfer_to_dict(data)
    if isinstance(data["value"]["top"], dict):
        top_value = data["value"]["top"]["value"]
    else:
        top_value = data["value"]["top"]
    if isinstance(data["value"]["bottom"], dict):
        bottom_value = data["value"]["bottom"]["value"]
    else:
        bottom_value = data["value"]["bottom"]
    return f"\\displaystyle \\stackrel{{{top_value}}}{{{bottom_value}}}"


def to_vector(data):
    data = transfer_to_dict(data)
    value = data["value"]
    return f"\\displaystyle \\vec{{{value}}}"


def to_font(data):
    data = transfer_to_dict(data)
    color = data["color"]
    value = data["value"]
    return f"\\displaystyle \\textcolor{{{color}}}{{{value}}}"


def to_img(data):
    data = transfer_to_dict(data)
    # 检查是否有svg路径
    src = data["value"].get("svg_src") or data["value"].get("src")

    # Todo： func
    if src and not src.startswith("http"):
        src = "http://www.leleketang.com" + src
    
    width = data["value"]["width"]
    height = data["value"]["height"]

    # 在实际LaTeX文档中，您可能需要提前设置图片的路径
    # \graphicspath{{<path to your images>}}
    return f"\\includegraphics[width={width}px,height={height}px]{{{src}}}"


def to_list(data):
    data = transfer_to_dict(data)
    items = data["value"]
    latex_lines = []

    for item in items:
        tag = item["tag"]
        content = item["content"]
        # 连接标签和内容
        latex_lines.append(f"{tag}{content}")

    return "\\\\".join(latex_lines)


def to_matrix(data):
    data = transfer_to_dict(data)
    matrix = data["value"]
    latex_rows = []

    for row in matrix:
        latex_rows.append(" & ".join(row))

    return "\\begin{bmatrix} " + " \\\\ ".join(latex_rows) + " \\end{bmatrix}"


def to_richtext(data):
   
    data = transfer_to_dict(data)
    value = data["value"]
    if value["type"] == "merge":
        print("merge")
        merged_items = value["value"]
        latex_parts = []
        for item in merged_items:

            if item["type"] == "string":
                latex_parts.append(to_string(item))
            elif item["type"] == "script":
                latex_parts.append(script_to_latex(item))
            # 这里可以添加其他数据类型的处理
        return "".join(latex_parts)
    else:
        # 对于其他可能的rich text内容，可以在这里添加处理方法
        pass


def to_select(data):
    print(f"start select")
    # data = transfer_to_dict(data)
    # 获取必要的数据
    options = data["options"]
    print(f"solution")
    solution = data.get("solution", "")
    print(solution)

    no = data.get("no", "")

    # 开始组装题目文本
    output = []

    # 添加题号
    # output.append(f"**{no}.** (多选)")
    
    print(f"option is {options}")

    # 循环遍历所有选项，并添加到输出中
    for opt in options:
        tag = opt["tag"]
        from ..converter.converter import converter, c2
        print(f"opt is {opt},, type is {type(opt)}")
        option_text = c2(opt["option"])
        output.append(f"{tag}. {option_text}")

    # 添加答案
    # output.append(f"\n**答案**: {solution}")

    return " [br]".join(output)


def to_multiselect(data):
    data = transfer_to_dict(data)
    # 获取必要的数据
    options = data["options"]
    solution = data["solution"]
    no = data.get("no", "")

    # 开始组装题目文本
    output = []

    # 添加题号（如果有）
    if no:
        output.append(f"**{no}.**")

    # 循环遍历所有选项，并添加到输出中
    for opt in options:
        tag = opt["tag"]
        option_text = opt["option"]
        output.append(f"{tag}. {option_text}")

    # 添加答案
    solution_str = "、".join(solution)
    output.append(f"\n**答案**: {solution_str}")

    return "\n".join(output)


def to_table(data):
    data = transfer_to_dict(data)
    rows = data["value"]

    # 开始组装LaTeX表格
    output = ["\\begin{tabular}{|c|c|}"]
    output.append("\\hline")

    for row in rows:
        row_content = []
        for cell in row:
            value = cell["value"]
            # 如果值是一个列表，则处理列表内容
            if isinstance(value, list):
                value = process_cell_content(value)
            # 根据colspan和rowspan添加相应的指令
            cell_content = f"\\multicolumn{{{cell['colspan']}}}{{|c|}}{{\\multirow{{{cell['rowspan']}}}{{*}}{{{value}}}}}"
            row_content.append(cell_content)
        output.append(" & ".join(row_content) + " \\\\ \\hline")

    output.append("\\end{tabular}")

    return "\n".join(output)


def process_cell_content(cell_content):
    data = transfer_to_dict(data)
    processed = []
    for item in cell_content:
        if "type" in item:
            if item["type"] == "colorpink":
                processed.append(f"\\textcolor{{pink}}{{{item['value']}}}")
            elif item["type"] == "script":
                front = item["value"].get("front")
                sup = item["value"].get("sup")
                processed.append(f"{front}$^{{{sup}}}$")
        else:
            processed.append(item)
    return "".join(processed)


def to_text(data):
    data = transfer_to_dict(data)
    # 获取填空内容

    # 如果value为空，使用blank代替
    
    value = "______"

    # 判断是否支持换行
    if data.get("wordwrap"):
        value = "\\newline ".join(value.split("\n"))
    # 判断是否适配填空长度
    if not data.get("noautowidth"):
        value = "\\underline{\\hspace{" + str(round(len(value) * 0.8, 2)) + "cm}}"
    else:
        value = "\\underline{" + value + "}"

    return value


def to_bold(data):
    data = transfer_to_dict(data)
    return f"\\textbf{{{data['value']}}}"


def to_italic(data):
    data = transfer_to_dict(data)
    return f"\\textit{{{data['value']}}}"


def to_unit(data):
    print("unit")
    data = transfer_to_dict(data)
    return f"\\mathrm{{{data['value']}}}"

def to_merge(data):
    from ..converter.converter import converter
    merged_items = data["value"]
    latex_parts = []
    print("mmm")

    for item in merged_items:
        print(f"latex is {latex_parts}")
        print(f"merge item is {item}")
        if isinstance(item, list):
            print(f"list it {item}")
            for i in item:
                

                
                if isinstance(i, str):
                    latex_parts.append(i)
                    continue
                else:
                    latex_parts.append(converter(i))
            continue

        if isinstance(item, str):
            print(f"str is {item}")
            latex_parts.append(item)
            continue
        # if item["type"] == "string":
        latex_parts.append(converter(item))

        # elif item["type"] == "script":
            # latex_parts.append(script_to_latex(item))
        # 这里可以添加其他数据类型的处理

    try:
        dd = "".join(latex_parts)
    except Exception as e:
        print(latex_parts)
        print(f"he")
        print(e)
    final = "".join(latex_parts)
    print(f"finally get {final}")
    return final


def to_wave(data):
    return data["value"]

functions_list  = [
    to_latex,
    to_string,
    to_arc,
    to_arrow,
    to_arrow_down,
    to_equation,
    to_fraction,
    to_latexarray,
    to_latexwrap,
    to_limit,
    to_average,
    to_repeater,
    to_repeating,
    to_root,
    to_script,
    to_sum,
    to_underbrace,
    to_underline,
    to_valence,
    to_vector,
    to_font,
    to_img,
    to_list,
    to_matrix,
    to_richtext,
    to_select,
    to_multiselect,
    to_table,
    to_text,
    to_bold,
    to_italic,
    to_unit,
    to_merge,
    to_wave
]

conversion_functions = {func.__name__[3:]: func for func in functions_list}



from ..utils.text_to_latex  import conversion_functions
from ..utils import transfer_to_dict
from ujson import JSONDecodeError
from typing import Union

def converter(data: Union[dict, str]):
    if isinstance(data, list):
        res = []
        for d in data:
            converter(d)

    try:
        content_dict = transfer_to_dict(data)
    except (JSONDecodeError):
        # print("load err")
        # print(type(data))
        return data
    question = content_dict["caption"] if "caption" in content_dict else content_dict
    
    answer = content_dict.get("answer", None)
    types = (question['type'])
    
    print(f"type is {types}")
    
    try:
        if types =="wave":
            print(f"wave")
            print(conversion_functions.get(types))
        conversion_func = conversion_functions.get(types)
        print(f"con func is {conversion_func}")
        if conversion_func:
            print(f"yes")
            # maybe \\newline
            # return conversion_func(te).replace("[br]", "<br />")
            question_part = conversion_func(question)
        else:
            
            raise ValueError(f"No conversion function for type {types} found.")
    except Exception as e:
        #  print(f"error for {type(data)}")
        #  print(e)
        pass
    
    if not answer:
        # return 'error'
        try: 
            
            return question_part
        except:
            return "error"
    else:
        if answer_converter(answer):
            return question_part + answer_converter(answer)
        return question_part
    

def answer_converter(answer: dict):
    print(f"answer is {answer}")
    try:
        answer_dict = transfer_to_dict(answer)
    except (JSONDecodeError):
        # print("load err")
        # print(type(data))
        return answer

    
    if answer_dict:
        types = answer.get("type", None)
        print(f"answer type is {types}")
        try:
            conversion_func = conversion_functions.get(types)
            print(f"type is {types}")
            if conversion_func:
                # maybe \\newline
                # return conversion_func(te).replace("[br]", "<br />")
                print(f"start")
                print(f"ans dict is {answer_dict}")
                answer_part = conversion_func(answer_dict)
                return answer_part
            else:
                
                raise ValueError(f"No conversion function for type {types} found.")
        except Exception as e:
                #  print(f"error for {type(data)}")
                #  print(e)
            pass
        return []
    

def is_final_object(obj):
    if isinstance(obj, dict):
        print(f"is dict")
        for key, val in obj.items():
            if isinstance(val, dict):
                if 'type' in val:
                    return False
                if not is_final_object(val):
                    return False
    elif isinstance(obj, list):
        for item in obj:
            if isinstance(item, dict):
                if 'type' in item:
                    return False
                if not is_final_object(item):
                    return False
    return True
    

def render(obj):
    print(f"start re")
    if isinstance(obj, dict) and 'type' in obj and obj['type']:
        obj_type = obj['type'].lower()
        conversion_func = conversion_functions.get(obj_type)
        if conversion_func:
            print(f"{obj_type}", obj)
            # maybe \\newline
            # return conversion_func(te).replace("[br]", "<br />")
            question_part = conversion_func(obj)
            print(f"fi ")
            print(question_part)
            return question_part
        
    elif isinstance(obj, str):
        print(f"str is {obj}")
        return obj.replace("[br]", "\n")
    elif isinstance(obj, list):
        return " ".join(obj)
    else:
        return "Invalid object type"

def to_old_version(obj):
    if isinstance(obj, dict):
        if 'type' in obj:
            if obj['type'] in ['merge', 'equation', 'latexarray']:
                nobjs = [to_old_version(o) for o in obj['value']]
                obj['value'] = nobjs
                return obj
            elif obj['type'] in ['select', 'multiselect']:
                nobjs = [to_old_version(o) for o in obj['options']]
                obj['options'] = nobjs
                return obj
            elif obj['type'] in ['table', 'matrix', 'grid']:
                for col in obj['value']:
                    for i, row in enumerate(col):
                        col[i] = to_old_version(row)
                return obj
            else:
                for k, val in obj.items():
                    obj[k] = to_old_version(val)
                return obj
        else:
            nobjs = {}
            is_list = all(isinstance(k, int) for k in obj.keys())
            for k, o in obj.items():
                nobjs[k] = to_old_version(o)
            if is_list:
                return {'type': 'merge', 'value': list(nobjs.values())}
            else:
                return nobjs
    elif isinstance(obj, list):
        # 对列表中的每个值递归调用
        print("list")
        print([to_old_version(val) for val in obj])
        print(f"s")
        return [to_old_version(val) for val in obj]
    else:
        return obj


    
def c2(obj):
    if not obj:
        return ""
    print(f"obj type is {type(obj)}")
    # if not isinstance(obj, dict):
    #     try:
    #         obj = transfer_to_dict(obj)
    #     except (JSONDecodeError):
    #         # print("load err")
    #         # print(type(data))
    #         return obj 
    print(f"obj is {obj}")
    obj = to_old_version(obj)
    print(f"obj is {obj}")
    
    while True:
        current_obj = obj
        if is_final_object(obj):
            return render(obj)
        print(f"not final")
        found = False
        if isinstance(obj, dict):
            for key, val in current_obj.items():
                print("keys is", key)
                if not isinstance(val, dict):
                    continue
                if is_final_object(val):
                    print(f"true")
                    if 'type' in val:
                        print(f"key is {key}")
                        current_obj[key] = render(val)
                        print(f"cu is {current_obj}")
            else:
                current_obj = val
                found = True
                break
        elif isinstance(obj, list):
            l = len(obj)
            print(f"len is {l}")
            for i in range(l):
                current_obj[i] = render(current_obj[i])
            print(f"final obj i is {current_obj}")
                


        if not found:
            break

        if current_obj is obj:
            break
    print(f"type if {type(current_obj)}")
    print(f"obj before render is {current_obj}")
    print(current_obj)
    
    return render(current_obj)

def c_by_type(data: dict):
    print(f"data is  {data}")
    
    data_type = data.get("type")
    solution = transfer_to_dict(data.get("solution", ""))
    content = transfer_to_dict(data.get("content"))
    explains = transfer_to_dict(data.get("explains"))
    knowledge_point = data.get("knowledge_point", None)
    answer = content.get("answer", None)
    # Check for empty solutions in certain question types
    if data_type in [1, 2, 3] and not solution:
        errorMsg('填空答案错误')
        return

    # Check for empty content in single and multiple choice questions
    if data_type in [2, 3] and ('answer' not in content or not answer):
        errorMsg('单选题干错误')
        return

    # Initialize the structure to store the processed data
    json_data = {'type': data_type, 'solution': None, "caption": None}


    json_data["caption"] = c2(content["caption"])
    json_data["knowledge_point"] = knowledge_point

    if data_type == 1 or data_type =="填空":
        # Fill-in-the-blank
        answer_index = get_answer_index(content['caption'])
        answer_index_count = len(answer_index)

        json_data['solution'] = []
        for sk, s in enumerate(solution):
            if sk > (answer_index_count - 1):
                continue

            # if not validate_solution(s, qid):
            #     return

            # Process the solution based on its type
            # json_data['solution'].append(render_solution(s))# 
            # Todo: add render 
            json_data['solution'].append(s)

    elif data_type == 2:
        # Single choice
        print(solution)
        print(type(solution))
        if not solution[0]['value']:
            errorMsg('单选题答案为空')
            return

        json_data['option'] = c2(content['answer'])
        json_data['solution'] = solution[0]['value']

    elif data_type == 3:
        # Multiple choice
        if not solution[0]['value']:
            errorMsg('多选题答案为空')
            return

        json_data['option'] = c2(content['answer'])
        json_data['solution'] = solution[0]['value']

    elif data_type == 4:
        # Custom question type handling
        qq = {'type': data_type, 'content': content, 'solution': solution, 'explains': explains}
        
        splits = ttl_question_show_answer(qq)

        if solution and splits is False:
            print(f"json data is {json_data}")
            errorMsg('答案拆分失败, c_b_type')
            return

        # json_data['solution'] = process_splits_solution(splits)
        
        # Todo: add render
        json_data['solution'] = splits

    else:
        errorMsg('题目type错误')
        return
    
    invalid = ['略', '详见答案']
    try:
        ss = explains.get('解答设置')
    except:
        print("Ex is ",explains)
    if explains.get('解答设置') == 'same':
        json_data['explain'] = ''
    else:
        jieda = explains.get('解答')
        if jieda:
            jieda = jieda.strip() if isinstance(jieda, str) else jieda
            if isinstance(jieda, str) and jieda in invalid:
                json_data['explain'] = ''
            else:
                # 假设 LatexRender.to_html 是一个已定义的函数，用来转换 LaTeX 字符串为 HTML
                json_data['explain'] = c2(jieda)
        else:
            json_data['explain'] = ''

    return json_data

    
    
def errorMsg(msg, qid=1):
    print(f"Error in question {qid}: {msg}")

    

def get_answer_index(caption):
        if isinstance(caption, dict):
            index = []
            if 'index' in caption and 'type' in caption:
                index.append(caption)
            else:
                for key, value in caption.items():
                    if isinstance(value, dict) and 'index' in value and 'type' in value:
                        index.append(value)
                    elif isinstance(value, (dict, list)):
                        temp = get_answer_index(value)
                        if temp:
                            index.extend(temp)
            return index if index else False

        elif isinstance(caption, list):
            index = []
            for item in caption:
                if isinstance(item, dict) and 'index' in item and 'type' in item:
                    index.append(item)
                elif isinstance(item, (dict, list)):
                    temp = get_answer_index(item)
                    if temp:
                        index.extend(temp)
            return index if index else False
        
        
def ttl_question_show_answer(question, q_type=1, mode=0):
        if question['type'] != 4:
            return False

        splits = split_question(question['type'], question['content'], question['solution'], question['explains'], True, mode)
        if splits is False:
            return False

        return splits


def split_question(q_type, content, solution, explains, flag, mode):
    # Implementation of split_question method
    pass


def c_by_type_new(data: dict):
    print(f"data is  {data}")
    
    data_type = data.get("type")
    solution = transfer_to_dict(data.get("solution", ""))
    content = transfer_to_dict(data.get("content"))
    explains = transfer_to_dict(data.get("explain"))
    knowledge_point = data.get("knowledge_point", None)
    difficulty = data.get("difficulty", None)
    answer = content.get("answer", None)
    # Check for empty solutions in certain question types
    if data_type in [1, 2, 3] and not solution:
        errorMsg('填空答案错误')
        return

    # Check for empty content in single and multiple choice questions
    if data_type in [2, 3] and ('answer' not in content or not answer):
        errorMsg('单选题干错误')
        return

    # Initialize the structure to store the processed data
    json_data = {'type': data_type, 'solution': None, "caption": None}


    json_data["caption"] = c2(content["caption"])
    json_data["knowledge_point"] = knowledge_point
    json_data["difficulty"] = difficulty
    
    

    if data_type == 1:
        # Fill-in-the-blank
        answer_index = get_answer_index(content['caption'])
        answer_index_count = len(answer_index)

        json_data['solution'] = []
        for sk, s in enumerate(solution):
            if sk > (answer_index_count - 1):
                continue

            # if not validate_solution(s, qid):
            #     return

            # Process the solution based on its type
            # json_data['solution'].append(render_solution(s))# 
            # Todo: add render 
            json_data['solution'].append(s)

    elif data_type == 2:
        # Single choice
        print(solution)
        print(type(solution))
        if not solution[0]['value']:
            errorMsg('单选题答案为空')
            return

        json_data['option'] = c2(content['answer'])
        json_data['solution'] = solution[0]['value']

    elif data_type == 3:
        # Multiple choice
        if not solution[0]['value']:
            errorMsg('多选题答案为空')
            return

        json_data['option'] = c2(content['answer'])
        json_data['solution'] = solution[0]['value']

    elif data_type == 4:
        # Custom question type handling
        qq = {'type': data_type, 'content': content, 'solution': solution, 'explains': explains}
        
        splits = ttl_question_show_answer(qq)

        if solution and splits is False:
            print(f"ddd")
            print(f"json data is {json_data}")
            errorMsg('答案拆分失败')
            return

        # json_data['solution'] = process_splits_solution(splits)
        
        # Todo: add render
        json_data['solution'] = splits

    else:
        print(f"data type is {data_type}")
        errorMsg('题目type错误')
        return
    
    invalid = ['略', '详见答案']
    try:
        ss = explains.get('解答设置')
    except:
        print("Ex is ",explains)
    if explains.get('解答设置') == 'same':
        json_data['explain'] = ''
    else:
        jieda = explains.get('解答')
        if jieda:
            jieda = jieda.strip() if isinstance(jieda, str) else jieda
            if isinstance(jieda, str) and jieda in invalid:
                json_data['explain'] = ''
            else:
                # 假设 LatexRender.to_html 是一个已定义的函数，用来转换 LaTeX 字符串为 HTML
                json_data['explain'] = c2(jieda)
        else:
            json_data['explain'] = ''

    return json_data


import ujson


def transfer_to_dict(data: dict | str):
    if isinstance(data, dict):
        return data
    if isinstance(data, list):
        return data
    if  not data:
        return None
    return ujson.loads(data)


from setuptools import setup, find_packages

setup(
    name="latex_formatter",
    version="1.1",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    install_requires=[
        "ujson >= 5.8.0",
        # 你的依赖包列表，例如：
        # "matplotlib >= 2.2.0"
    ],
)
